/***************************************************************************
                          irmcsyncconfig.cpp  -  description
                             -------------------
    begin                : Sat Feb 14 2003
    copyright            : (C) 2003 by Simone Gotti
    email                : simone.gotti@email.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "irmcsyncconfig.h"

#include "irmcsynckonnector.h"

#include <kapplication.h>
#include <kdebug.h>
#include <klocale.h>
#include <kdialog.h>

#include <qcombobox.h>
#include <qlabel.h>
#include <qradiobutton.h>
#include <qbuttongroup.h> 


using namespace KBluetooth;

IrMCSyncConfig::IrMCSyncConfig( QWidget *parent, const char *name )
		: KRES::ConfigWidget( parent, name )
{
	initUI();
}

IrMCSyncConfig::~IrMCSyncConfig()
{}

void IrMCSyncConfig::loadSettings( KRES::Resource *resource )
{
	kdDebug() << "IrMCSyncConfig::loadSettings()" << endl;

	KSync::ThreadedPlugin *k =
	    dynamic_cast<KSync::ThreadedPlugin *>( resource );
	if ( !k ) {
		kdError() << "IrMCSyncConfig::loadSettings(): Wrong Konnector type." << endl;
		return;
	}

	kdDebug() << "k->deviceBtAddress() =" << k->deviceBtAddress() << endl;
	m_cmbDeviceBtAddress->insertItem (k->deviceBtAddress(), 0 );
	m_chkSyncCalendar->setChecked( k->syncCalendar() );
	m_chkSyncAddressBook->setChecked( k->syncAddressBook() );
	
}

void IrMCSyncConfig::saveSettings( KRES::Resource *resource )
{
	kdDebug() << "IrMCSyncConfig::saveSettings()" << endl;
	KSync::ThreadedPlugin *k =
	    dynamic_cast<KSync::ThreadedPlugin *>( resource );
	if ( !k ) {
		kdError() << "IrMCSyncConfig::loadSettings(): Wrong Konnector type." << endl;
		return;
	}

	k->setDeviceBtAddress( m_cmbDeviceBtAddress->currentText() );
	k->setSyncCalendar( m_chkSyncCalendar->isChecked() );
	k->setSyncAddressBook( m_chkSyncAddressBook->isChecked() );
}

void IrMCSyncConfig::initUI()
{
	m_layout = new QGridLayout( this, 4, 2, KDialog::marginHint(),
                              KDialog::spacingHint() );
	
	m_buttonGroup = new QButtonGroup( 1, Qt::Vertical, this);
	m_buttonGroup->setExclusive(true);

	m_lblDeviceBtAddress = new QLabel( i18n("Bluetooth device:"), this );

	m_cmbDeviceBtAddress = new QComboBox(this);
	m_cmbDeviceBtAddress->setEditable( true );

	m_serviceSelector = new ServiceSelectionWidget(this, "0x1104", true, true, false);

	m_chkSyncCalendar = new QRadioButton (i18n("Sync calendar"), m_buttonGroup);
	
	m_chkSyncAddressBook = new QRadioButton (i18n("Sync addressBook"), m_buttonGroup);
	
	// Set The Layout
	m_layout->addWidget( m_lblDeviceBtAddress, 0, 0 );
	m_layout->addWidget( m_cmbDeviceBtAddress, 0, 1 );

	m_layout->addMultiCellWidget( m_serviceSelector, 1, 1, 0, 1 );
	connect(m_serviceSelector, SIGNAL(serviceChanged (KBluetooth::DeviceAddress, uint)), SLOT(slotServiceChanged (KBluetooth::DeviceAddress, uint)));

  m_layout->addMultiCellWidget( m_buttonGroup, 2, 2, 0, 1 );
  m_buttonGroup->layout()->add( m_chkSyncCalendar );
  m_buttonGroup->layout()->add( m_chkSyncAddressBook );
}

void IrMCSyncConfig::slotServiceChanged(KBluetooth::DeviceAddress btAddr, uint)
{
	m_cmbDeviceBtAddress->insertItem (QString(btAddr), 0 );
}

#include "irmcsyncconfig.moc"
