/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation; either version 2 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *****************************************************************************/

#ifndef METRONOME_UI_H
#define METRONOME_UI_H

#include <kpushbutton.h>
#include <qtimer.h>
#include <kmessagebox.h>
#include <kconfig.h>
#include <kstatusbar.h>
#include <kapplication.h>
#include <kdebug.h>

#include "metronomewidget.h"

int meter = 0;
int beat = 0;
bool sound = false;
bool easter = false;

QTimer *t;
QTimer *deactTimer;

void metronomeWidget::init() {
    int interval;
    t = new QTimer(this);
    deactTimer = new QTimer(this);

    // define the frequencies for the two beeps.
    frequency1 = 880.0;
    frequency2 = 440.0;

    server = ka.server();

    // should it just force sound to off?
    if (server.isNull()) {
	    KMessageBox::sorry(this, "Can't connect to sound server",
			    "Sound server unavailable");
	    exit(0);
    }


    connect(t, SIGNAL(timeout()), this, SLOT(timeout()));
    connect(deactTimer, SIGNAL(timeout()), this, SLOT(beepOff()));

    interval = getInterval(tempoDial->value());
    if (meter == 0) {
	beatSlider->setValue(2);
    } else if (meter == 1) {
	beatSlider->setValue(3);
    } else if (meter == 2) {
	beatSlider->setValue(4);
    } else if (meter == 3) {
	beatSlider->setValue(5);
    } else {
	beatSlider->setValue(6);
    }

    // Create the synth objects.  must do it from the server for it to work.
    play = Arts::DynamicCast(server.createObject("Arts::Synth_AMAN_PLAY"));
    freq = Arts::DynamicCast(server.createObject("Arts::Synth_FREQUENCY"));
    sin = Arts::DynamicCast(server.createObject("Arts::Synth_WAVE_SIN"));
    adsr = Arts::DynamicCast(server.createObject("Arts::Synth_ENVELOPE_ADSR"));

    // initializing the play object is required.
    play.title("klicker");
    play.autoRestoreID("klicker");
    
    Arts::setValue(freq, frequency2);
    // these values were carefully chosen.  If you MUST alter them, make sure
    // that they total around 100ms, or you will have sound problems.
    Arts::setValue(adsr, "attack", 0.030);
    Arts::setValue(adsr, "decay", 0.030);
    Arts::setValue(adsr, "sustain", 0.010);
    Arts::setValue(adsr, "release", 0.030);
    Arts::setValue(adsr, "active", 0.0);
    Arts::connect(freq, sin);
    Arts::connect(sin, adsr, "invalue");
    Arts::connect(adsr, "outvalue", play, "left");
    Arts::connect(adsr, "outvalue", play, "right");

    // start the objects.  This is, of course, different from actually
    // generating the sound - that is done by bringing adsr.active high.
    freq.start();
    sin.start();
    adsr.start();
    play.start();

    readProperties(kapp->config());

}

void metronomeWidget::destroy() {

        kdDebug() << "destroy called\n" << endl;

	freq.stop();
	sin.stop();
	adsr.stop();
	play.stop();

	Arts::disconnect(adsr, "outvalue", play, "right");
	Arts::disconnect(adsr, "outvalue", play, "left");
	Arts::disconnect(freq, sin);
	Arts::disconnect(sin, adsr, "invalue");

	server = Arts::SoundServerV2::null();

	this->saveProperties(kapp->config());

}

void metronomeWidget::saveProperties(KConfig *config) {

	config->setGroup("klicker");

       config->writeEntry("Meter1", meter);
        config->writeEntry("SoundEnabled", sound);
        config->writeEntry("EasterEnabled", easter);
        config->writeEntry("Speed", tempoDial->value());

        config->sync();
}

void metronomeWidget::readProperties(KConfig *config) {

	config->setGroup("klicker");

    meter = config->readNumEntry("Meter1", 2);
    sound = config->readBoolEntry("SoundEnabled", false);
    easter = config->readBoolEntry("EasterEnabled", false);
    tempoDial->setValue(config->readNumEntry("Speed", 30));

    beatSlider->setValue(meter + 2);

    emit signalConfigChanged();

}
void metronomeWidget::beepOff() {

	// the beep is turned off by setting active low.
	Arts::setValue(adsr, "active", 0.0);
	emit signalChangeStatusBar((const QString &)QString::QString(""));
}

void metronomeWidget::enableSound() {

	sound = true;
	saveProperties(kapp->config());
}

void metronomeWidget::disableSound() {

	sound = false;
	Arts::setValue(adsr, "active", 0.0);
	saveProperties(kapp->config());
}

void metronomeWidget::enableEaster() {

	easter = true;
	saveProperties(kapp->config());
}

void metronomeWidget::disableEaster() {

	easter = false;
	saveProperties(kapp->config());

}



int metronomeWidget::getInterval(int i)
{
    int interval;

    interval = 60000;	// number of milliseconds in a minute.
    interval /= i;		// beats per minute, but expressed in milliseconds
    return interval;

}

void metronomeWidget::changeInterval(int i)
{
    bool a;
    int interval;
    interval = getInterval(i);
    a = t->isActive();
    t->changeInterval(interval);
    if (!a) {
	t->stop();
    }
    tempoLCD->display(i);
    tempoDial->setValue(i);

    saveProperties(kapp->config());
}

void metronomeWidget::timeout()
{
    beat++;
    if (beat >= meter + 2) {
	beat = 0;
    }
    if (beat == 0) {
	Arts::setValue(freq, frequency1);
	reset();
	outLED1->on();
    } else if (beat == 1) {
	Arts::setValue(freq, frequency2);
	reset();
	outLED2->on();
    } else if (beat == 2) {
	Arts::setValue(freq, frequency2);
	reset();
	outLED3->on();
    } else if (beat == 3) {
	Arts::setValue(freq, frequency2);
	reset();
	outLED4->on();
    } else if (beat == 4) {
	Arts::setValue(freq, frequency2);
	reset();
	outLED5->on();
    } else if (beat == 5) {
	Arts::setValue(freq, frequency2);
	reset();
	outLED6->on();
    }
   
    
    if (sound == true) {

	    Arts::setValue(adsr, "active", 1.0);

	    // this is connected elsewhere to beepOff.
	    // don't set it for less than 100ms, or you risk not having the
	    // sound show up at all.
	    deactTimer->start(100, true);
    }
    if (easter == true) {
	    if (beat == 1) {
	emit signalChangeStatusBar((const QString &)QString::QString("Boop"));
	    } else {
	emit signalChangeStatusBar((const QString &)QString::QString("Beep"));
	    }
    }
}

void metronomeWidget::start()
{
    t->start(getInterval(tempoDial->value()), false);
}

void metronomeWidget::stop()
{
    t->stop();
    beat = 0;
   reset();
}

void metronomeWidget::reset()
{
    outLED1->off();
    outLED2->off();
    outLED3->off();
    outLED4->off();
    outLED5->off();
    outLED6->off();
}
    
#endif


void metronomeWidget::changedBeat( int  tbeat)
{

    if (tbeat == 2) {
	outLED3->hide();
	outLED4->hide();
	outLED5->hide();
	outLED6->hide();
    } else if (tbeat == 3) {
	outLED3->show();
	outLED4->hide();
	outLED5->hide();
	outLED6->hide();
    } else if (tbeat == 4) {
	outLED3->show();
	outLED4->show();
	outLED5->hide();
	outLED6->hide();
    } else if (tbeat == 5) {
	outLED3->show();
	outLED4->show();
	outLED5->show();
	outLED6->hide();
    } else {
	outLED3->show();
	outLED4->show();
	outLED5->show();
	outLED6->show();
    }
 
    meter = tbeat - 2;
    beat = 0;
    reset();
}
