/***************************************************************************
 *                                                                         *
 *   Derived from the linux-2.6 tree                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef PWMANAGER_COMPILER_H_
#define PWMANAGER_COMPILER_H_

#ifdef _deprecated_
# undef _deprecated_
#endif
#ifdef noinline
# undef noinline
#endif
#ifdef always_inline
# undef always_inline
#endif
#ifdef must_check_result
# undef must_check_result
#endif

#if __GNUC__ >= 4
# define _deprecated_		__attribute__((deprecated))
# define noinline		__attribute__((noinline))
# define always_inline		inline __attribute__((always_inline))
# define must_check_result	__attribute__((warn_unused_result))
#elif __GNUC__ == 3
# if __GNUC_MINOR__ >= 1
#  define _deprecated_		__attribute__((deprecated))
#  define noinline		__attribute__((noinline))
#  define always_inline		inline __attribute__((always_inline))
# endif
# if __GNUC_MINOR__ >= 4
#  define must_check_result	__attribute__((warn_unused_result))
# endif
#elif __GNUC__ == 2
# if __GNUC_MINOR__ < 96
#  ifdef __builtin_expect
#   undef __builtin_expect
#  endif
#  define __builtin_expect(x, expected_value) (x)
# endif
#else
# error "Sorry, your compiler is too old/not supported."
#endif

/* Trap pasters of __FUNCTION__ at compile-time */
#if __GNUC__ > 2 || __GNUC_MINOR__ >= 95
# define __FUNCTION__ (__func__)
#endif

/*
 * Allow us to mark functions as 'deprecated' and have gcc emit a nice
 * warning for each use, in hopes of speeding the functions removal.
 * Usage is:
 * 		int _deprecated_ foo(void)
 */
#ifndef _deprecated_
# define _deprecated_		/* unimplemented */
#endif

#ifndef must_check_result
# define must_check_result
#endif

#ifndef noinline
# define noinline
#endif

#ifndef always_inline
# define always_inline	inline
#endif

/* define likely() and unlikely() */
#ifdef likely
# undef likely
#endif
#ifdef unlikely
# undef unlikely
#endif
#define likely(x)	__builtin_expect(!!(x), true)
#define unlikely(x)	__builtin_expect(!!(x), false)


#ifdef NOREGPARM
# undef NOREGPARM
#endif
#ifdef REGPARM
# undef REGPARM
#endif

#ifdef __i386__
# define NOREGPARM	__attribute__((regparm(0)))
# define REGPARM	__attribute__((regparm(3)))
#endif

#ifndef NOREGPARM
# define NOREGPARM	/* nothing */
#endif
#ifndef REGPARM
# define REGPARM	/* nothing */
#endif

#endif // PWMANAGER_COMPILER_H_
