/***************************************************************************
 *                                                                         *
 *   copyright (C) 2004, 2005  by Michael Buesch                           *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef PWM_CONFIGURATION_H_
#define PWM_CONFIGURATION_H_

#define CONF_DEFAULT_PWTIMEOUT		10		/* 10 sec */
#define CONF_DEFAULT_LOCKTIMEOUT	0		/* 0 == disable */
#define CONF_DEFAULT_TRAY		true
#define CONF_DEFAULT_UNLOCKONOPEN	true
#define CONF_DEFAULT_COMPRESSION	0x01		/* gzip */
#define CONF_DEFAULT_CRYPTALGO		(0x01 - 1)	/* blowfish */
#define CONF_DEFAULT_HASHALGO		(0x01 - 1)	/* sha1 */
#define CONF_DEFAULT_AUTOMINIMIZE	false
#define CONF_DEFAULT_BROWSERCOMMAND	""
#define CONF_DEFAULT_XTERMCOMMAND	"konsole -e"
#define CONF_DEFAULT_FILEPERMISSIONS	0600
#define CONF_DEFAULT_MAKEFILEBACKUP	false
#define CONF_DEFAULT_AUTODEEPLOCK	true
#define CONF_DEFAULT_KWALLETEMU		true
#define CONF_DEFAULT_MINIMIZELOCK	true
#define CONF_DEFAULT_NEWENTRLOCKSTAT	false		/* new entries unlocked */
#define CONF_DEFAULT_WNDCLOSE		true		/* don't minimize to tray */
#define CONF_DEFAULT_EXTGENERATOR	"makepasswd --length 10"
#define CONF_DEFAULT_REVEAL_AFTER_GEN	true

/** This is just because I'm too lazy to always
  * type this loooong statement, when accessing
  * configuration parameters.
  */
#define conf()	Configuration::obj()

#include <kstandarddirs.h>
#include <qstring.h>
#include <qsize.h>
#include <qfont.h>
#include <qwidget.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlineedit.h>

#include <kconfigskeleton.h>

#include "pwmexception.h"
#include "globalstuff.h"

class QLabel;
class QGroupBox;
class QPushButton;
class QResizeEvent;

/** New global configuration file interface
  * using KDE-3.2 KConfigSkeleton model.
  */
class Configuration
{
public:
	Configuration();
	~Configuration();

	static Configuration * obj()
		{
			PWM_ASSERT(_obj);
			return _obj;
		}
	static void init()
		{
			PWM_ASSERT(!_obj);
			_obj = new Configuration;
		}
	static void cleanup()
			{ delete_ifnot_null(_obj); }


	/** Read the configuration from the file.
	  * Normally this function has not to be called manually.
	  */
	void readConfig()
			{ skel->readConfig(); }
	/** Write the configuration to the file.
	  * Normally this function has not to be called manually.
	  */
	void writeConfig()
			{ skel->writeConfig(); }
	/** reset the whole configuration to its defaults */
	void resetConfig()
			{ skel->setDefaults(); }
	/** show the configuration window */
	bool showConfWnd(QWidget *parent = 0);

public:
	/* functions for reading the configuration settings */
	/* GLOBAL */
	QString confGlobAutoStart()
			{ return cGlobAutoStart; }
	QString confGlobBrowserCommand()
			{ return cGlobBrowserCommand; }
	QString confGlobXtermCommand()
			{ return cGlobXtermCommand; }
	QString confGlobExtGenerator()
			{ return cGlobExtGenerator; }
	QFont confGlobEntryFont()
			{ return cGlobEntryFont; }
	int confGlobPwTimeout()
			{ return cGlobPwTimeout; }
	int confGlobLockTimeout()
			{ return cGlobLockTimeout; }
	int confGlobCompression()
			{ return cGlobCompression; }
	int confGlobCryptAlgo()
			{ return cGlobCryptAlgo + 1; }
	int confGlobHashAlgo()
			{ return cGlobHashAlgo + 1; }
	int confGlobFilePermissions()
			{ return cGlobFilePermissions; }
	bool confGlobMinimizeLock()
			{ return cGlobMinimizeLock; }
	bool confGlobUnlockOnOpen()
			{ return cGlobUnlockOnOpen; }
	bool confGlobTray()
			{ return cGlobTray; }
	bool confGlobMakeFileBackup()
			{ return cGlobMakeFileBackup; }
	bool confGlobAutoDeepLock()
			{ return cGlobAutoDeepLock; }
	bool confGlobKwalletEmu()
			{ return cGlobKwalletEmu; }
	bool confGlobNewEntrLockStat()
			{ return cGlobNewEntrLockStat; }
	bool confGlobRevealAfterGen()
			{ return cGlobRevealAfterGen; }
	/* WND */
	QSize confWndMainWndSize()
			{ return cWndMainWndSize; }
	bool confWndAutoMinimizeOnStart()
			{ return cWndAutoMinimizeOnStart; }
	bool confWndClose()
			{ return cWndClose; }

public:
	/* functions for writing the configuration settings */
	/* GLOBAL */
	void confGlobAutoStart(const QString &e)
			{ cGlobAutoStart = e; }
	void confGlobBrowserCommand(const QString &e)
			{ cGlobBrowserCommand = e; }
	void confGlobXtermCommand(const QString &e)
			{ cGlobXtermCommand = e; }
	void confGlobExtGenerator(const QString &e)
			{ cGlobExtGenerator = e; }
	void confGlobEntryFont(const QFont &e)
			{ cGlobEntryFont = e; }
	void confGlobPwTimeout(int e)
			{ cGlobPwTimeout = e; }
	void confGlobLockTimeout(int e)
			{ cGlobLockTimeout = e; }
	void confGlobCompression(int e)
			{ cGlobCompression = e; }
	void confGlobCryptAlgo(int e)
			{ cGlobCryptAlgo = e - 1; }
	void confGlobHashAlgo(int e)
			{ cGlobHashAlgo = e - 1; }
	void confGlobFilePermissions(int e)
			{ cGlobFilePermissions = e; }
	void confGlobMinimizeLock(bool e)
			{ cGlobMinimizeLock = e; }
	void confGlobUnlockOnOpen(bool e)
			{ cGlobUnlockOnOpen = e; }
	void confGlobTray(bool e)
			{ cGlobTray = e; }
	void confGlobMakeFileBackup(bool e)
			{ cGlobMakeFileBackup = e; }
	void confGlobAutoDeepLock(bool e)
			{ cGlobAutoDeepLock = e; }
	void confGlobKwalletEmu(bool e)
			{ cGlobKwalletEmu = e; }
	void confGlobNewEntrLockStat(bool e)
			{ cGlobNewEntrLockStat = e; }
	void confGlobRevealAfterGen(bool e)
			{ cGlobRevealAfterGen = e; }
	/* WND */
	void confWndMainWndSize(const QSize &e)
			{ cWndMainWndSize = e; }
	void confWndAutoMinimizeOnStart(bool e)
			{ cWndAutoMinimizeOnStart = e; }
	void confWndClose(bool e)
			{ cWndClose = e; }

protected:
	/** initialize the skeleton */
	void initSkel();

protected:
	/** static instance of this class returned by obj() */
	static Configuration *_obj;
	/** main configuration access skeleton */
	KConfigSkeleton *skel;

protected:
	/* configuration variables. All prefixed with 'c'. */
	/* GLOBAL */
	QString cGlobAutoStart;
	QString cGlobBrowserCommand;
	QString cGlobXtermCommand;
	QString cGlobExtGenerator;
	QFont cGlobEntryFont;
	int cGlobPwTimeout;
	int cGlobLockTimeout;
	int cGlobCompression;
	int cGlobCryptAlgo;
	int cGlobHashAlgo;
	int cGlobFilePermissions;
	bool cGlobMinimizeLock;
	bool cGlobUnlockOnOpen;
	bool cGlobTray;
	bool cGlobMakeFileBackup;
	bool cGlobAutoDeepLock;
	bool cGlobKwalletEmu;
	bool cGlobNewEntrLockStat;
	bool cGlobRevealAfterGen;
	/* WND */
	QSize cWndMainWndSize;
	bool cWndAutoMinimizeOnStart;
	bool cWndClose;
};

/* Big fat note: Internal stuff follows.
 * ============  Don't use the following classes outside of
 *		 the Configuration code.
 */

/** class for input of octal numbers (for example file permissions) */
class OctLineEdit : public QLineEdit
{
	Q_OBJECT
	Q_OVERRIDE( QString text READ text WRITE setText )

public:
	OctLineEdit(QWidget *parent, const char *name = 0);
	~OctLineEdit();

	void setText(const QString &t);
	QString text() const;

protected:
	void keyPressEvent(QKeyEvent *e);
};

/** global configuration page */
class ConfPageGlobal : public QWidget
{
public:
	ConfPageGlobal(QWidget *parent = 0, const char *name = 0, WFlags f = 0);
};

/** configuration page for look&feel */
class ConfPageLookNFeel : public QWidget
{
public:
	ConfPageLookNFeel(QWidget *parent = 0, const char *name = 0, WFlags f = 0);
};

/** file configuration page */
class ConfPageFile : public QWidget
{
public:
	ConfPageFile(QWidget *parent = 0, const char *name = 0, WFlags f = 0);
};

/** timeouts configuration page */
class ConfPageTimeouts : public QWidget
{
public:
	ConfPageTimeouts(QWidget *parent = 0, const char *name = 0, WFlags f = 0);
};

/** configuration page for external apps */
class ConfPageExtApps : public QWidget
{
public:
	ConfPageExtApps(QWidget *parent = 0, const char *name = 0, WFlags f = 0);
};

/** autostart configuration page */
class ConfPageAutostart : public QWidget
{
	Q_OBJECT
public:
	ConfPageAutostart(QWidget *parent = 0, const char *name = 0, WFlags f = 0);

protected slots:
	void browseButton_slot();

protected:
	QLineEdit *kcfg_autoStart;
};

#endif // __CONFIGURATION_H
