/****************************************************************************
**
** Base64 code is derived from qt-x11-opensource-4.0.0-b1 by
**   Michael Buesch <mbuesch@freenet.de>
** Selftest code is written by
**   Michael Buesch <mbuesch@freenet.de>
**
** Base64 code is
**   Copyright (C) 1992-2004 Trolltech AS. All rights reserved.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include "libpwmanager/base64.h"
#include "pwmexception.h"
#include "globalstuff.h"


Base64::Base64()
{
	static bool mustSelfTest = true;
	if (mustSelfTest) {
		if (!selfTest()) {
			printError("BASE64 selftest failed!");
			BUG();
		}
		mustSelfTest = false;
	}
}

void Base64::encode(QByteArray *to, const QByteArray &from)
{
	const char alphabet[] = "ABCDEFGH" "IJKLMNOP" "QRSTUVWX" "YZabcdef"
				"ghijklmn" "opqrstuv" "wxyz0123" "456789+/";
	const char padchar = '=';
	int padlen = 0;
	unsigned int i = 0, fromSize = from.size();
	char *fromData = from.data();

	to->resize(((fromSize * 4) / 3) + 3);
	char *out = to->data();
	while (i < fromSize) {
		int chunk = 0;
		chunk |= int(uchar(fromData[i++])) << 16;
		if (i == fromSize)
			padlen = 2;
		else {
			chunk |= int(uchar(fromData[i++])) << 8;
			if (i == fromSize)
				padlen = 1;
			else
				chunk |= int(uchar(fromData[i++]));
		}
		int j = (chunk & 0x00fc0000) >> 18;
		int k = (chunk & 0x0003f000) >> 12;
		int l = (chunk & 0x00000fc0) >> 6;
		int m = (chunk & 0x0000003f);
		*out++ = alphabet[j];
		*out++ = alphabet[k];
		if (padlen > 1)
			*out++ = padchar;
		else
			*out++ = alphabet[l];
		if (padlen > 0)
			*out++ = padchar;
		else
			*out++ = alphabet[m];
	}
	to->truncate(out - to->data());
}

void Base64::decode(QByteArray *to, const QByteArray &from)
{
	unsigned int buf = 0;
	unsigned int i, offset = 0, nbits = 0;
	unsigned int fromSize = from.size();

	to->resize((fromSize * 3) / 4);
	for (i = 0; i < fromSize; ++i) {
		int ch = from.at(i);
		int d;

		if (ch >= 'A' && ch <= 'Z')
			d = ch - 'A';
		else if (ch >= 'a' && ch <= 'z')
			d = ch - 'a' + 26;
		else if (ch >= '0' && ch <= '9')
			d = ch - '0' + 52;
		else if (ch == '+')
			d = 62;
		else if (ch == '/')
			d = 63;
		else
			d = -1;

		if (d != -1) {
			buf = (buf << 6) | d;
			nbits += 6;
			if (nbits >= 8) {
				nbits -= 8;
				(*to)[offset++] = buf >> nbits;
				buf &= (1 << nbits) - 1;
			}
		}
	}
	to->truncate(offset);
}

bool Base64::selfTest()
{
	const char plain1[] = "Base64";
	const unsigned int plain1_len = array_size(plain1) - 1;
	const char plain2[] = "abcdefghijklmnopqrstuvwxyz";
	const unsigned int plain2_len = array_size(plain2) - 1;
	const char enc1[] = "QmFzZTY0";
	const unsigned int enc1_len = array_size(enc1) - 1;
	const char enc2[] = "YWJjZGVmZ2hpamtsbW5vcHFyc3R1dnd4eXo=";
	const unsigned int enc2_len = array_size(enc2) - 1;
	QByteArray plain, encoded;

	plain.duplicate(plain1, plain1_len);
	encode(&encoded, plain);
	if (memcmp(enc1, encoded, enc1_len))
		return false;
	plain.resize(0);
	decode(&plain, encoded);
	if (memcmp(plain1, plain, plain1_len))
		return false;

	plain.duplicate(plain2, plain2_len);
	encode(&encoded, plain);
	if (memcmp(enc2, encoded, enc2_len))
		return false;
	plain.resize(0);
	decode(&plain, encoded);
	if (memcmp(plain2, plain, plain2_len))
		return false;

	return true;
}
