/***************************************************************************
 *                                                                         *
 *   copyright (C) 2004, 2005  by Michael Buesch                           *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef LIBPWMANAGER_FILECRYPTO_H_
#define LIBPWMANAGER_FILECRYPTO_H_

#include <qcstring.h>


/** Main interface for encrypting/decrypting
  * a PwManager-file data stream.
  */
class FileCrypto
{
public:
	FileCrypto();
	~FileCrypto();

	/** pad the data up to the given boundary.
	  * "buf" has to be big enough!
	  */
	static void padData(unsigned char *buf,
			    size_t bufLen,
			    size_t boundary);
	/** unpad the data */
	static void unpadData(const unsigned char *buf,
			      size_t *bufLen);
	/** returns the total buffer length */
	static size_t getBufLen(size_t inBufLen, size_t boundary)
		{
			return ((boundary - (inBufLen % boundary)) + inBufLen);
		}

	/** Encrypt the data stream. */
	bool encrypt(QByteArray *data,
		     const QByteArray &passphrase,
		     unsigned char cryptAlgo,
		     unsigned char hashAlgo);
	/** Decrypt the data stream. */
	bool decrypt(QByteArray *data,
		     const QByteArray &passphrase,
		     unsigned char cryptAlgo,
		     unsigned char hashAlgo);
};

#endif // LIBPWMANAGER_FILECRYPTO_H_
