/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004, 2005  by Michael Buesch                     *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef PWMANAGER_GLOBALSTUFF_H_
#define PWMANAGER_GLOBALSTUFF_H_

#include "globalconfig.h"
#include "compiler.h"

#include <stdint.h>

#include <iostream>

using std::cout;
using std::endl;

#ifdef PROG_NAME
# undef PROG_NAME
#endif
#define PROG_NAME	"PwManager"

#ifdef PACKAGE_NAME
# undef PACKAGE_NAME
#endif
#define PACKAGE_NAME	"pwmanager"

#ifdef PACKAGE_VER
# undef PACKAGE_VER
#endif
#define PACKAGE_VER	"1.1.0pre3"

#ifdef CONFIG_DEBUG
# define PWM_DEBUG
#else
# undef PWM_DEBUG
#endif

#ifdef QT_MAKE_VERSION
# undef QT_MAKE_VERSION
#endif
#define QT_MAKE_VERSION(a,b,c)	(((a) << 16) | ((b) << 8) | (c))

/** remove "unused parameter" warnings */
#ifdef PARAM_UNUSED
# undef PARAM_UNUSED
#endif
#define PARAM_UNUSED(x)	(void)x

/** return the number of elements in an array */
#ifdef array_size
# undef array_size
#endif
#define array_size(x)	(sizeof(x) / sizeof((x)[0]))


#include <qstring.h>
#include <qcstring.h>


class QWidget;

namespace PwManager
{

class PwMGlobal
{
private:
	PwMGlobal() { }

public:
	/** convert a byte stream to human readable HEX-notation. */
	static QString humanReadableHex(const QByteArray &d);
};

/** delete the memory and NULL the pointer */
template<class T> inline
void delete_and_null(T *&p)
{
	delete p;
	p = 0;
}
/** delete the memory if the pointer isn't a NULL pointer */
template<class T> inline
void delete_ifnot_null(T *&p)
{
	if (p)
		delete_and_null(p);
}

template<class T> inline
void delete_and_null_array(T *&p)
{
	delete [] p;
	p = 0;
}

template<class T> inline
void delete_ifnot_null_array(T *&p)
{
	if (p)
		delete_and_null_array(p);
}

} // namespace PwManager

#endif // PWMANAGER_GLOBALSTUFF_H_
