/***************************************************************************
 *                                                                         *
 *   copyright (C) 2004 by Michael Buesch                                  *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "ipc.h"


#define LOOKUP_TIMER_FREQ	5 /* msec */


ThreadIpcHost::ThreadIpcHost(bool start)
{
	connect(&timer, SIGNAL(timeout()),
		this, SLOT(queueLookup()));
	if (start)
		startLookup();
}

ThreadIpcHost::~ThreadIpcHost()
{
	if (timer.isActive())
		stopLookup();
}

void ThreadIpcHost::startLookup()
{
	timer.start(LOOKUP_TIMER_FREQ);
}

void ThreadIpcHost::stopLookup()
{
	timer.stop();
	queueLookup();
}

void ThreadIpcHost::queueMessage(const QByteArray &msg)
{
	QByteArray tmp;
	tmp.duplicate(msg);
	mqLock.lock();
	mq.append(tmp);
	mqLock.unlock();
}

void ThreadIpcHost::queueLookup()
{
	mqLock.lock();
	QValueList<QByteArray>::iterator i = mq.begin(),
					 end = mq.end();
	for ( ; i != end; ++i)
		emit receivedMessage(*i);
	mq.clear();
	mqLock.unlock();
}


ThreadIpcClient::ThreadIpcClient(ThreadIpcHost &_host)
 : host (&_host)
{
}

ThreadIpcClient::~ThreadIpcClient()
{
}

void ThreadIpcClient::sendMessage(const QByteArray &msg)
{
	host->queueMessage(msg);
}

#include "ipc.moc"
