/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004, 2005  by Michael Buesch                     *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "pwmexception.h"

#include <kmessagebox.h>

#include <signal.h>


using namespace std;


#if COMPILE_BUGCHECKING != 0
bool _pwmanagerFatalInProgress = false;

void _pwmanagerFatal(unsigned int id,
		     const char *file,
		     int line,
		     const char *additional)
{
	bool shutdownApp = false;

	_pwmanagerFatalInProgress = true;
	cerr << PROG_NAME "-" PACKAGE_VER ": ";
	switch (id) {
	case 0:
		cerr << "BUG";
		shutdownApp = true;
		break;
	case 1:
		cerr << "badness";
		break;
	case 2:
		cerr << "TODO";
		break;
	case 3:
		cerr << "FIXME";
		break;
#ifdef PWM_DEBUG
	case 4:
		cerr << "PWM_ASSERT failed";
		break;
	default:
		cerr << "ARGHHH!";
		break;
#endif // PWM_DEBUG
	}
	cerr << " at " << file << ":" << line;
	if (additional)
		cerr << " (" << additional << ")";
	cerr << endl;

	if (id == 0) {
		KMessageBox::error(0, QString(PROG_NAME "-" PACKAGE_VER
					": Detected a BUG at ")
				      + file + ":" + QString::number(line)
				      + "\nPlease report this to:\n"
				        "Michael Buesch <mbuesch@freenet.de>\n"
				        "Thank you.",
				   PROG_NAME "-" PACKAGE_VER " BUG detected");
	}
	if (shutdownApp) {
		// try to enter the posix signal handler, to exit cleanly.
#if defined(__GNUC__) && defined(__i386__)
		// this is the "undefined instruction" on i386.
		__asm__ __volatile__("ud2");
#else
		if (raise(SIGILL)) {
			/* hm, failed. This should never happen, but
			 * just in case, shut down hard here.
			 */
			::exit(PWM_EXIT_BUG);
		}
#endif
	}
	_pwmanagerFatalInProgress = false;
}
#endif // COMPILE_BUGCHECKING != 0

void _pwmanagerPrintError(const QString &msg)
{
	QString _msg(PROG_NAME " generated a fatal fault:\n");
	_msg += msg;
	cerr << "\n\n" << _msg.ascii() << endl;
	KMessageBox::error(0, _msg, PROG_NAME " fatal ERROR!");
}

void _pwmanagerPrintInfo(const QString &msg)
{
	cout << PROG_NAME " INFO: " << msg.ascii() << endl;
}

void _pwmanagerPrintWarn(const QString &msg)
{
	cerr << PROG_NAME " WARNING: " << msg.ascii() << endl;
}

#ifdef PWM_DEBUG
void _pwmanagerPrintDebug(const QString &msg)
{
	cout << PROG_NAME " DEBUG: " << msg.ascii() << endl;
}
#endif // PWM_DEBUG
