/***************************************************************************
 *                                                                         *
 *   copyright (C) 2004, 2005  by Michael Buesch                           *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef LIBPWMANAGER_SERIALIZER_H_
#define LIBPWMANAGER_SERIALIZER_H_

#include "libpwmanager/simplexmlparser.h"

#include <qcstring.h>
#include <qstring.h>
#include <qdatetime.h>


namespace PwManager
{

class DataRoot;
class DataMeta;
class DataItem;
class DataCategory;
class DataEntry;

/** Class for (de)serializing the XML stream read from the file */
class Serializer
{
public:
	/** construct an empty serializer document */
	Serializer();
	/** destructor */
	~Serializer();

	/** serialize the PwMDataItem and store it as XML tree (internal) */
	bool serialize(QByteArray *buffer,
		       const DataRoot &data);
	/** deserialize the (parsed) XML tree and return it */
	bool deSerialize(const QByteArray &buffer,
			 DataRoot *data);

protected:
	/** parsed data tree. */
	std::vector<SimpleXmlParser::XmlElement> *tree;
	/** xml data. */
	QCString *xmlData;
	/** current DataRoot. */
	DataRoot *curRoot;

protected:
	/** encode entry data. */
	QCString encode(const QByteArray &s);
	/** decode entry data. */
	void decode(std::string *s);

	/** deserialize the contents of the category */
	bool readCategory(SimpleXmlParser::XmlElement *e,
			  DataItem *d);
	/** deserialize a data item */
	bool readEntry(SimpleXmlParser::XmlElement *e,
		       DataItem *d);
	/** deserialize the meta data */
	bool readMeta(SimpleXmlParser::XmlElement *e,
		      DataMeta *d);

	bool writeCategory(const DataCategory &d);
	bool writeEntry(const DataEntry &d);
	bool writeMeta(const DataMeta &d);

	/* functions to generate the XML tags. */
	void tagBegin_root(const QCString &version);
	void tagBegin_cat(const QCString &refCnt,
			  const QCString &id);
	void tagBegin_entry(const QCString &refCnt,
			    const QCString &id);
	void tagBegin_meta();
	void tag_field(const QCString &refCnt,
		       const QCString &id,
		       const QCString &type,
		       const QCString &content);
	void tag_metaCreate(const QDateTime &date);
	void tag_metaValid(const QDateTime &date);
	void tag_metaExpire(const QDateTime &date);
	void tag_metaUpdate(const QDateTime &date);
	void tag_metaUpdateInt(unsigned long updateInt);
	void tag_metaUniqueId(unsigned int uniqueid);
	void tagEnd_root();
	void tagEnd_cat(const QCString &refCnt);
	void tagEnd_entry(const QCString &refCnt);
	void tagEnd_meta();
#if 0
protected:
	/** XML data tree document */
	QDomDocument *domDoc;

protected:
	/** check if this is valid PwManager XML data */
	bool checkValid();
	/** deserialize the contents of the category */
	bool readCategory(const QDomNode &n,
			  DataItem *data,
			  DataRoot *root);
	/** deserialize a data item */
	bool readEntry(const QDomNode &n,
		       DataItem *data,
		       DataRoot *root);
	/** deserialize the meta data */
	bool readMeta(const QDomNode &n,
		      DataMeta *data);
	/** serialize the contents of a category. */
	bool writeCategory(const DataCategory *data,
			   QDomElement *e);
	/** serialize the contents of an entry. */
	bool writeEntry(const DataEntry *data,
			QDomElement *e);
	/** serialize the meta data. */
	bool writeMeta(const DataMeta *data,
		       QDomElement *e);
#endif
};

} // namespace PwManager

#endif // LIBPWMANAGER_SERIALIZER_H_
