/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004, 2005  by Michael Buesch                     *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   sha1.c - SHA1 hash function                                           *
 *	Copyright (C) 1998, 2001, 2002 Free Software Foundation, Inc.      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef LIBPWMANAGER_SHA1_H_
#define LIBPWMANAGER_SHA1_H_

#include <stdint.h>
#include <qcstring.h>

typedef uint8_t		byte;

#define SHA1_HASH_LEN_BIT	160
#define SHA1_HASH_LEN_BYTE	(SHA1_HASH_LEN_BIT / 8)

/** sha1 hash algorithm.
  * Derived from libgcrypt-1.1.12
  */
class Sha1
{
	struct SHA1_CONTEXT
	{
		uint32_t  h0,h1,h2,h3,h4;
		uint32_t  nblocks;
		byte buf[64];
		int  count;
	};

public:
	Sha1();

	void sha1_init();
	void sha1_write(const byte *inbuf, uint32_t inlen);
	void sha1_read(QByteArray *ret);

protected:
	void sha1_final();
	void burn_stack (int bytes);
	void transform(const byte *data);
	bool selfTest();

	/** Rotate a 32 bit integer by n bytes */
	uint32_t rol(uint32_t x, int n)
	{
#if defined(__GNUC__) && defined(__i386__)
		__asm__("roll %%cl,%0"
			:"=r" (x)
			:"0" (x),"c" (n));
		return x;
#else
		return ((x) << (n)) | ((x) >> (32-(n)));
#endif
	}

protected:
	struct SHA1_CONTEXT ctx;
};

#endif // LIBPWMANAGER_SHA1_H_
