/***************************************************************************
 *                                                                         *
 *  Simple XML Parser.                                                     *
 *     copyright (C)  Tim Hockin <thockin@hockin.org>                      *
 *                                                                         *
 *  Heavily modified / ported to C++                                       *
 *     copyright (C) 2004, 2005  Michael Buesch <mbuesch@freenet.de>       *
 *                                                                         *
 * Released under the LGPL version 2.1 (http://www.gnu.org)                *
 * This program is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU Lesser General Public License version 2.1 *
 * as published by the Free Software Foundation.                           *
 *                                                                         *
 ***************************************************************************/

#ifndef LIBPWMANAGER_SIMPLEXMLPARSER_H_
#define LIBPWMANAGER_SIMPLEXMLPARSER_H_

/* enable/disable parser debugging (0/1) */
#define SIMPLEXMLPARSER_DEBUG		0

#include <string>
#include <vector>
#include <map>


/** This is a very simple and fast XML parser.
  * It does not parse every XML code, but it parses
  * the PwManager XML Code good and fast. And that's what
  * we want. No need to add more code here to enable parsing
  * of other code.
  *
  * @author Tim Hockin
  * @author Michael Buesch <mbuesch@freenet.de>
  */
class SimpleXmlParser
{
public:
	struct XmlElement
	{
		std::string				name;
		std::string				content;
		std::map<std::string, std::string>	props;
		std::vector<XmlElement>			children;
	};

protected:
	enum state_t
	{
		ST_TAGSTART = 0,
		ST_START_STAG,
		ST_STAG_NAME_DONE,
		ST_PROPNAME,
		ST_GETEQ,
		ST_GOTEQ,
		ST_PROPVAL,
		ST_SQ_PROPVAL,
		ST_POST_PROPVAL,
		ST_END_SETAG,
		ST_END_STAG,
		ST_CONTENT,
		ST_MAYBE_COMMENT,
		ST_NEWTAG_START,
		ST_START_ETAG,
		ST_ETAG_NAME,
		ST_END_ETAG
	};

public:
	SimpleXmlParser() { }

	bool parseBuffer(const std::string &buf,
			 std::vector<XmlElement> *ret);
#if SIMPLEXMLPARSER_DEBUG != 0
	std::string dumpElements(const std::vector<XmlElement> &e,
				 int indent = 0);
#endif // SIMPLEXMLPARSER_DEBUG

protected:
	bool parseElement(char first,
			  XmlElement *element);
	void prolog();
	void comment();
	char unescape();
	bool nextChar(char *c);
	inline bool isNameStart(char c);
	inline bool isName(char c);
	void saveProp(XmlElement *e,
		      std::string *propname,
		      std::string *propval);

protected:
#if SIMPLEXMLPARSER_DEBUG != 0
	unsigned int _lineno;
	void setLineno(unsigned int i)
			{ _lineno = i; }
	unsigned int getLineno()
			{ return _lineno; }
	void incLineno()
			{ _lineno++; }
#else // SIMPLEXMLPARSER_DEBUG
	inline void setLineno(unsigned int)
			{ /* nothing */ }
	inline void incLineno()
			{ /* nothing */ }
#endif // SIMPLEXMLPARSER_DEBUG

protected:
	unsigned int bufPos;
	unsigned int bufLen;
	const std::string *buffer;
};

#endif // LIBPWMANAGER_SIMPLEXMLPARSER_H_
