/***************************************************************************
 *                                                                         *
 *   copyright (C) 2004 by Michael Buesch                                  *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef PWMANAGER_MASTERKEY_SMARTKEYBACKEND_H_
#define PWMANAGER_MASTERKEY_SMARTKEYBACKEND_H_

#include <qobject.h>
#include <qthread.h>
#include <qcstring.h>


class SmartKey;
class WaitWnd;
class CTCardTrader;
class CTCard;
class ThreadIpcHost;

class SmartKey_backend_ : public QThread
			, public QObject
{
	friend class SmartKey_backend;

protected:
	SmartKey_backend_() { }
};

class SmartKey_backend : public SmartKey_backend_
{
	Q_OBJECT

public:
	enum Task
	{
		task_none,
		task_cancel,

		task_read,
		task_write,
		task_erase,
	};
	enum Status
	{
		status_none,
		status_running,
		status_cancel,

		status_ok,
		status_errNoMem,
		status_errOpen,
		status_errTimeout,
		status_errRead,
		status_errWrite,
		status_err2small,
	};

public:
	SmartKey_backend(SmartKey *_sk);
	~SmartKey_backend();

	Status writeCard(const QByteArray &data);
	Status readCard(QByteArray *data);
	Status eraseCard();

public slots:
	void cancel();

protected:
	/** overload start(). */
	void start(Priority priority = InheritPriority);
	/** fire up the thread and wait for the result. */
	Status fireUp();
	/** sit back and relax for a while. */
	void relax();

	/** thread worker function. */
	void run();
	/** write data to the card. */
	void write(const QByteArray &data);
	/** read data from the card. */
	void read(QByteArray *data);
	/** erase the card. */
	void erase();

	/** open the card. */
	bool ct_open(CTCardTrader **_ctTrader, CTCard **_ctCard);
	/** close the card. */
	bool ct_close(CTCardTrader *ctTrader, CTCard *ctCard);
	/** write data to the card. */
	bool ct_write(const QByteArray &data, CTCard *ctCard);
	/** read data from the card. */
	bool ct_read(QByteArray *data, CTCard *ctCard);
	/** erase the card. */
	bool ct_erase(CTCard *ctCard);

protected slots:
	/** new IPC message received from the thread. */
	void threadMessage(const QByteArray &msg);

protected:
	/** parent. */
	SmartKey *sk;
	/** wait window for opening the card. */
	WaitWnd *openWaitWnd;
	/** wait window progress counter. */
	unsigned int openWaitWndCnt;
	/** wait window for accessing the card. */
	WaitWnd *accessWaitWnd;
	/** host IPC message queue. */
	ThreadIpcHost *hostIpc;
};

#endif // PWMANAGER_MASTERKEY_SMARTKEYBACKEND_H_
