/***************************************************************************
 *                                                                         *
 *   copyright (C) 2004, 2005  by Michael Buesch                           *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef PWMINIT_H_
#define PWMINIT_H_

#include <qvaluelist.h>
#include <qobject.h>

#include "pwmexception.h"
#include "libpwmanager/libpwmanager.h"

#define pwminit		PwMInit::self()

class DCOPClient;
class KWalletEmu;
class KApplication;
class QWidget;
class PwMTray;
class PwMDoc;
class PwMApplication;


/** PwManager initialisation class.
  * This class sets up a new instance of PwManager and finally
  * starts the user interface.
  */
class PwMInit : public QObject
{
	Q_OBJECT
	friend class PwMApplication;

protected:
	enum RunStatus
	{
		unknown,
		init,
		running,
		shutdown
	};

	/* saved command line options. */
	class savedCmd_t
	{
	public:
		bool minimized;
		bool minToTray;
		// initial open paths are not saved here.
	};

public:
	PwMInit(PwMApplication *_app);
	~PwMInit();

	/** Returns a pointer to the PwMInit instance. */
	static PwMInit * self()
			{ return myself; }

	/** Copy text to the global X11 clipboard. */
	static void copyToClipboard(const QString &text);
	/** Generate a new random password and display
	  * it in a window. If parent is NULL, we use init()->curWidget().
	  */
	static void genPassword(QWidget *parent = 0);


	/** Initialize the application. */
	bool initializeApp();
	/** Shutdown the application.
	  * You can call this function from every context. It will do its
	  * best to clean up everything.
	  * This function may return if forceHardShutdown is false.
	  */
	void shutdownApp(int exitStatus = PWM_EXIT_SUCCESS,
			 bool forceHardShutdown = false);

	/** Returns a pointer to the LibPwManager object. */
	PwManager::LibPwManager * lib()
			{ return _lib; }
	/** returns a pointer to the PwMApplication object. */
	PwMApplication * getApp()
			{ return app; }
	/** returns a pointer to the current widget */
	QWidget * curWidget()
			{ return lib()->getParentWidget(); }
	/** returns a pointer to the dcop client */
	DCOPClient * dcopClient()
			{ return _dcopClient; }
	/** returns a pointer to the KWallet emulation */
	KWalletEmu * kwalletEmu()
			{ return _kwalletEmu; }
	/** returns a pointer to the tray icon */
	PwMTray * tray()
			{ return _tray; }

	/** (re)initialize the KWallet emulation */
	void initKWalletEmu(bool forceDisable = false, bool forceReload = false);
	/** (re)initialize the tray icon */
	void initTray();

protected:
	/** initialize the DCOP connection */
	void initDCOP();
	/** initialize posix signal handling */
	void initPosixSignalHandler();
	/** handle the commandline args */
	void handleCmdLineArgs(bool initial = true);
	/** minimize all open mainWnds to tray (or if
	  * no tray is present, minimize them the normal way)
	  */
	void minimizeAllMainWnd(bool toTray);

protected slots:
	/** the tray icon got closed */
	void trayIconClosed(PwMTray *tray);
	/** remove the tray and quit the app */
	void removeTrayAndQuit();

protected:
	/** pointer to the one and only PwMInit instance. */
	static PwMInit *myself;
	/** LibPwManager instance. */
	PwManager::LibPwManager *_lib;
	/** run-status of the app */
	RunStatus runStatus;
	/** Application pointer */
	PwMApplication *app;
	/** pointer to the dcop client */
	DCOPClient *_dcopClient;
	/** pointer to the KWallet emulation */
	KWalletEmu *_kwalletEmu;
	/** pointer to the tray icon */
	PwMTray *_tray;
	/** saved command line options. */
	savedCmd_t savedCmd;
};

#endif // PWMINIT_H_
