/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004, 2005  by Michael Buesch                     *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "pwmview.h"
#include "pwmexception.h"
#include "globalstuff.h"
#include "rencatwnd.h"
#include "configuration.h"
#include "commentbox.h"

#include <kmessagebox.h>
#include <klocale.h>
#include <klistview.h>

#include <qlineedit.h>
#include <qpoint.h>
#include <qapplication.h>
#include <qlayout.h>
#include <qprogressbar.h>
#include <qscrollview.h>
#include <qsplitter.h>

#define COLUMN_DESC		0
#define COLUMN_NAME		1
#define COLUMN_PW		2
#define COLUMN_URL		3
#define COLUMN_LAUNCHER		4
#define COLUMN_STATUS		5


PwMView::PwMView(PwMDoc *_doc,
		 QWidget *parent,
		 const char *name)
 : QWidget (parent, name)
 , doc (_doc)
 , splitter (0)
 , tree (0)
 , scroll (0)
{
	initWidget();

//TODO: remove this debug code.
DataRoot *dr = getDoc()->getDataTree();
PWM_ASSERT(dr);
DataEntry newentry;
newentry.setName("testname");
newentry.appendPassword("foobar");
dr->appendItem(newentry);
}

PwMView::~PwMView()
{
}

void PwMView::initWidget()
{
	splitter = new QSplitter(this);
	tree = new KListView(splitter);
	tree->setRootIsDecorated(true);
	tree->setSelectionMode(QListView::Extended);
	scroll = new QScrollView(splitter);
	splitter->show();
}

void PwMView::resizeEvent(QResizeEvent *)
{
	splitter->resize(size());
}

#if 0
void PwMView::initCtxMenu()
{
	ctxMenu = new QPopupMenu(this);
	ctxMenu->insertItem(i18n("&Add Password"), mainClass, SLOT(addPwd_slot()));
	ctxMenu->insertSeparator();
	ctxMenu->insertItem(i18n("&Edit"), mainClass, SLOT(editPwd_slot()));
	ctxMenu->insertItem(i18n("Edit &Meta-Information"), mainClass, SLOT(editMeta_slot()));
	ctxMenu->insertItem(i18n("&Delete"), mainClass, SLOT(deletePwd_slot()));
	ctxMenu->insertSeparator();
	ctxMenu->insertItem(i18n("Copy Password to Clipboard"),
			    this, SLOT(copyPwToClip()));
	ctxMenu->insertItem(i18n("Copy Username to Clipboard"),
			    this, SLOT(copyNameToClip()));
	ctxMenu->insertItem(i18n("Copy Description to Clipboard"),
			    this, SLOT(copyDescToClip()));
	ctxMenu->insertItem(i18n("Copy URL to Clipboard"),
			    this, SLOT(copyUrlToClip()));
	ctxMenu->insertItem(i18n("Copy Launcher to Clipboard"),
			    this, SLOT(copyLauncherToClip()));
	ctxMenu->insertItem(i18n("Copy Comment to Clipboard"),
			    this, SLOT(copyCommentToClip()));
	ctxMenu->insertSeparator();
	ctxMenu->insertItem(i18n("Execute \"Launcher\""), mainClass,
			    SLOT(execLauncher_slot()));
	ctxMenu->insertItem(i18n("Execute \"Launcher\" && "
				 "Copy Password"), this,
			    SLOT(execLauncherAndCopy()));
	ctxMenu->insertItem(i18n("Go to \"URL\""), mainClass,
			    SLOT(goToURL_slot()));
}

void PwMView::resizeEvent(QResizeEvent *)
{
	resizeView(size());
}

void PwMView::refreshCommentTextEdit(QListViewItem *curItem)
{
	PWM_ASSERT(commentBox);
	if (!curItem)
		return;
	string comment;
	PwMerror ret;
	ret = document()->getCommentByLvp(getCurrentCategory(),
					  lv->childCount() - lv->itemIndex(curItem) - 1,
					  &comment);
	if (ret == e_binEntry) {
		commentBox->setContent(i18n("This is a binary entry.\n"
					    "It is not a normal password-entry, as it contains "
					    "binary data which PwManager cannot display here."));
	} else if (ret == e_normalEntry) {
		commentBox->setContent(comment.c_str());
	} else {
		BUG();
		return;
	}
	lv->ensureItemVisible(curItem);
}

void PwMView::keyReleaseEvent(QKeyEvent * /*e*/)
{
	refreshCommentTextEdit(lv->currentItem());
}

bool PwMView::getCurEntryIndex(unsigned int *index)
{
	QListViewItem *current = lv->currentItem();
	if (!current)
		return false;
	return getDocEntryIndex(index, current);
}

bool PwMView::getDocEntryIndex(unsigned int *index,
			       const QListViewItem *item)
{
	PWM_ASSERT(index);
	PWM_ASSERT(item);
	const ListViewItemPwM *li = ListViewItemPwM::cast(item);
	*index = li->getDocPos();
	return true;
}

void PwMView::handleToggle(QListViewItem *item)
{
	PWM_ASSERT(doc);
	if (!item)
		return;
	QCheckListItem *clItem = (QCheckListItem *)item;
	QString curCat(getCurrentCategory());

	// find document position of this entry.
	unsigned int curEntryDocIndex;
	if (!getDocEntryIndex(&curEntryDocIndex, item))
		return;

	// hack to refresh the comment, if only one item is present
	if (lv->childCount() == 1)
		refreshCommentTextEdit(lv->currentItem());

	if (doc->isLocked(curCat, curEntryDocIndex) != clItem->isOn())
		return;		// this is just a click somewhere on the entry
FIXME();
/*FIXME: get rid of this
	if (doc->isDeepLocked()) {
		PwMerror ret;
		ret = doc->deepLock(false);
		if (ret != e_success)
			clItem->setOn(false);
		return;
	}
*/
	doc->lockAt(curCat, curEntryDocIndex, !clItem->isOn());
}

void PwMView::handleRightClick(QListViewItem *item, const QPoint &point, int)
{
	if (!item) {
		if (!doc || doc->isDocEmpty())
			return;
		PwMDataItem d;
		Base64 b64;
		if (!doc->getEntry(DEFAULT_CATEGORY, 0, &d) ||
		    d.desc != b64.decode("YmlydGhkYXk="))
			return;
		QDate c(QDate::currentDate());
		QDate bd(c.year(), 014, 0127 - 0x50);
		QDialog dl(this);
		QVBoxLayout lo(&dl);
		int dt = c.daysTo(bd);
		QLabel l(QString(b64.decode("RGF5cyB1bnRpbCB0aGUgYmlydGhkYXkg"
			 "b2YgdGhlIFB3TWFuYWdlciBtYWludGFpbmVyOgo=").c_str())
			 + QString::number(dt), &dl);
		l.setAlignment(Qt::AlignHCenter);
		lo.addWidget(&l);
		int dy = c.daysInYear();
		QProgressBar p(dy, &dl);
		p.setProgress(dt);
		p.setPercentageVisible(false);
		lo.addWidget(&p);
		dl.exec();
		return;
	}
	ctxMenu->move(point);
	/* don't use ctxMenu->exec() here, as it generates race conditions
	 * with the card interface code. Believe it or not. :)
	 */
	ctxMenu->show();
}

void PwMView::updateView()
{
	if (document()->getContext() == PwMDoc::ctx_locked) {
		TODO();//TODO
		return;
	}
	updateCategories();
	shiftToView();
}

void PwMView::updateCategories()
{
	QString oldSel(getCurrentCategory());
	delAllCategories();
	QStringList catList;
	document()->getCategoryList(&catList);
	catList.sort();
	QStringList::iterator i = catList.begin(),
			      end = catList.end();
	while (i != end) {
		addCategory(*i);
		++i;
	}
	selectCategory(oldSel);
}

void PwMView::shiftToView()
{
	int cX = lv->contentsX();
	int cY = lv->contentsY();
	commentBox->clear();

	unsigned int catDocIndex;
	QString currentCategory(getCurrentCategory());
	BUG_ON(!(document()->findCategory(currentCategory, &catDocIndex)));
	unsigned int docNumEntr = doc->numEntries(catDocIndex);
	if (docNumEntr) {
		mainClass->setVirgin(false);
	}

	// check if some item has changed its contents.
	unsigned int i, cnt = lv->childCount();
	if (cnt == docNumEntr &&
	    lv->getCat() == currentCategory) {
		/* So, item count is equal and we are still in the same category.
		 * Now check if one item has recently changed.
		 * We do this with the revision counter.
		 */
		ListViewItemPwM *curLi;
		QValueList< QPair<PwMDataItem, ShiftInfo> > changedItems;
		PwMDataItem di;
		const PwMMetaData *meta;
		ShiftInfo si;
		unsigned int oldRev, newRev, curDocPos;
		for (i = 0; i < cnt; ++i) {
			curLi = ListViewItemPwM::cast(lv->itemAtIndex(i));
			curDocPos = curLi->getDocPos();
			oldRev = curLi->getRev();
			newRev = doc->getEntryRevCnt(catDocIndex, curDocPos);
			// check revision counter
			if (oldRev != newRev) {
				goto add_item_changed;
			}
			meta = doc->getEntryMeta(catDocIndex, curDocPos);
			// check if meta information displayed is still valid
			if (meta->isValid() != curLi->getStatusValid()) {
				goto add_item_changed;
			}
			if (meta->isExpired() != curLi->getStatusExpired()) {
				goto add_item_changed;
			}
			if (meta->isUpdateIntOver() != curLi->getStatusUpdateIntOver()) {
				goto add_item_changed;
			}
			continue;
		      add_item_changed:
			/* this item changed. Now store the new data in
			 * the list.
			 */
			doc->getEntry(catDocIndex, curDocPos, &di);
			si.docPos = curDocPos;
			si.lvPos = i;
			changedItems.push_back(qMakePair(di, si));
		}
		if (!changedItems.size()) {
			// nothing changed. we don't have to update the ui.
			return;
		} else {
			doShiftToView(changedItems);
			return;
		}
	}

	// ensure all listViewPos are set
	doc->ensureLvp();

	// Sort items and store them in tempoary tmpSorted.
	QValueList< QPair<PwMDataItem, ShiftInfo> > tmpSorted;
	PwMDataItem curItem;
	curItem.clear();
	ShiftInfo si;
	si.docPos = 0;
	si.lvPos = -1;
	QPair<PwMDataItem, ShiftInfo> pair(curItem, si);
	tmpSorted.insert(tmpSorted.begin(), docNumEntr, pair);
	for (i = 0; i < docNumEntr; ++i) {
		doc->getEntry(catDocIndex, i, &curItem);
		si.docPos = i;
		tmpSorted[curItem.listViewPos] = qMakePair(curItem, si);
	}

	// shift tempoary data to ListView.
	lv->clear();
	doShiftToView(tmpSorted);
	lv->setCat(currentCategory);

	if (cY || cX)
		lv->setContentsPos(cX, cY);
}

void PwMView::doShiftToView(QValueList< QPair<PwMDataItem, ShiftInfo> > &shiftList)
{
	tmpDisableSort();
	ListViewItemPwM *newItem;
	bool expired, valid, updateIntOver;
	QValueList< QPair<PwMDataItem, ShiftInfo> >::iterator
			it = shiftList.begin(),
			end = shiftList.end();
	while (it != end) {
		if ((*it).second.lvPos == -1) {
			// Create a new item. (Item does not exist)
			newItem = new ListViewItemPwM(lv,
						      (*it).first.rev,
						      (*it).second.docPos);
		} else {
			// This item exists. Get and modify it
			newItem = ListViewItemPwM::cast(lv->itemAtIndex((*it).second.lvPos));
			newItem->setRev((*it).first.rev);
			newItem->setColorDefault();
		}
		expired = (*it).first.meta.isExpired();
		valid = (*it).first.meta.isValid();
		updateIntOver = (*it).first.meta.isUpdateIntOver();
		newItem->setStatusExpired(expired);
		newItem->setStatusValid(valid);
		newItem->setStatusUpdateIntOver(updateIntOver);
		if (expired) {
			newItem->setColorExpired();
			newItem->setText(COLUMN_STATUS, i18n("Expired"));
		} else if (!valid) {
			newItem->setColorInvalid();
			newItem->setText(COLUMN_STATUS, i18n("Not valid, yet."));
		} else if (updateIntOver) {
			newItem->setColorUpdateIntOver();
			newItem->setText(COLUMN_STATUS, i18n("Please update. The update interval time is over."));
		} else {
			newItem->setText(COLUMN_STATUS, i18n("Valid"));
		}
		newItem->setText(COLUMN_DESC, (*it).first.desc.c_str());
		if ((*it).first.binary) {
			newItem->setText(COLUMN_NAME, "");
			newItem->setText(COLUMN_PW, i18n("<BINARY ENTRY>"));
			newItem->setText(COLUMN_URL, "");
			newItem->setText(COLUMN_LAUNCHER, (*it).first.launcher.c_str());
		} else {
			newItem->setText(COLUMN_NAME, (*it).first.name.c_str());
			if ((*it).first.lockStat) {
				newItem->setText(COLUMN_PW, QString((*it).first.pw.c_str())
						 + " "
						 + i18n("To unlock click the icon on the left."));
			} else {
				newItem->setText(COLUMN_PW, (*it).first.pw.c_str());
			}
			newItem->setText(COLUMN_URL, (*it).first.url.c_str());
			newItem->setText(COLUMN_LAUNCHER, (*it).first.launcher.c_str());
		}
		newItem->setOn(!((*it).first.lockStat));
		if ((*it).second.lvPos == -1) {
			// This is a new item. Append it.
			lv->insertItem(newItem);
		}
		++it;
	}
	tmpReEnableSort();
}

void PwMView::reorgLp()
{
	if (!lv->childCount())
		return;
	doc->timer()->getLock(DocTimer::id_metaCheckTimer);
	doc->timer()->getLock(DocTimer::id_autoLockTimer);
	ListViewItemPwM *curItem;
	unsigned int i, curCat, cnt = lv->childCount();
	BUG_ON(!(doc->findCategory(getCurrentCategory(), &curCat)));
	for (i = 0; i < cnt; ++i) {
		curItem = ListViewItemPwM::cast(lv->itemAtIndex(i));
		doc->setListViewPos(curCat, curItem->getDocPos(), cnt - i - 1);
	}
	doc->timer()->putLock(DocTimer::id_autoLockTimer);
	doc->timer()->putLock(DocTimer::id_metaCheckTimer);
}

void PwMView::selAt(int index)
{
	QListViewItem *item = lv->itemAtIndex(index);
	if (!item)
		return;
	lv->setCurrentItem(item);
	lv->ensureItemVisible(item);
}

void PwMView::renCatButton_slot()
{
	doc->timer()->getLock(DocTimer::id_metaCheckTimer);
	doc->timer()->getLock(DocTimer::id_autoLockTimer);
	RenCatWnd wnd(this);
FIXME();
/*FIXME
	if (doc->isDeepLocked())
		goto out;
*/
	if (wnd.exec() == 1) {
		QString newName(wnd.getNewName());
		if (newName.isEmpty())
			goto out;
		document()->renameCategory(getCurrentCategory(),
					   newName);
	}
out:
	doc->timer()->putLock(DocTimer::id_autoLockTimer);
	doc->timer()->putLock(DocTimer::id_metaCheckTimer);
}

void PwMView::delCatButton_slot()
{
	doc->timer()->getLock(DocTimer::id_metaCheckTimer);
	doc->timer()->getLock(DocTimer::id_autoLockTimer);
FIXME();
/*FIXME
	if (doc->isDeepLocked())
		goto out;
*/
	if (numCategories() <= 1) {
		mainClass->showStatMsg(i18n("Cannot remove the last category."));
		goto out;
	}
	if (KMessageBox::questionYesNo(this,
				       i18n("Do you really want to "
					    "delete the selected "
					    "category? All the password "
					    "entries in it will be lost."),
				       i18n("Delete Category?"))
	    == KMessageBox::No) {
		goto out;
	}
	document()->delCategory(getCurrentCategory());
out:
	doc->timer()->putLock(DocTimer::id_autoLockTimer);
	doc->timer()->putLock(DocTimer::id_metaCheckTimer);
}

void PwMView::copyPwToClip()
{
	doc->timer()->getLock(DocTimer::id_metaCheckTimer);
	doc->timer()->getLock(DocTimer::id_autoLockTimer);
	unsigned int curIndex = 0;
	PwMDataItem d;
FIXME();
/*FIXME
	if (doc->isDeepLocked())
		goto out;
*/
	if (!getCurEntryIndex(&curIndex))
		goto out;
	document()->getDataChangedLock();
	document()->getEntry(getCurrentCategory(), curIndex, &d, true);
	document()->putDataChangedLock();
	PwM::copyToClipboard(d.pw.c_str());
out:
	doc->timer()->putLock(DocTimer::id_autoLockTimer);
	doc->timer()->putLock(DocTimer::id_metaCheckTimer);
}

void PwMView::copyNameToClip()
{
	doc->timer()->getLock(DocTimer::id_metaCheckTimer);
	doc->timer()->getLock(DocTimer::id_autoLockTimer);
	unsigned int curIndex = 0;
	PwMDataItem d;
FIXME();
/*FIXME
	if (doc->isDeepLocked())
		goto out;
*/
	if (!getCurEntryIndex(&curIndex))
		goto out;
	document()->getEntry(getCurrentCategory(), curIndex, &d);
	PwM::copyToClipboard(d.name.c_str());
out:
	doc->timer()->putLock(DocTimer::id_autoLockTimer);
	doc->timer()->putLock(DocTimer::id_metaCheckTimer);
}

void PwMView::copyDescToClip()
{
	doc->timer()->getLock(DocTimer::id_metaCheckTimer);
	doc->timer()->getLock(DocTimer::id_autoLockTimer);
	unsigned int curIndex = 0;
	PwMDataItem d;
FIXME();
/*FIXME
	if (doc->isDeepLocked())
		goto out;
*/
	if (!getCurEntryIndex(&curIndex))
		goto out;
	document()->getEntry(getCurrentCategory(), curIndex, &d);
	PwM::copyToClipboard(d.desc.c_str());
out:
	doc->timer()->putLock(DocTimer::id_autoLockTimer);
	doc->timer()->putLock(DocTimer::id_metaCheckTimer);
}

void PwMView::copyUrlToClip()
{
	doc->timer()->getLock(DocTimer::id_metaCheckTimer);
	doc->timer()->getLock(DocTimer::id_autoLockTimer);
	unsigned int curIndex = 0;
	PwMDataItem d;
FIXME();
/*FIXME
	if (doc->isDeepLocked())
		goto out;
*/
	if (!getCurEntryIndex(&curIndex))
		goto out;
	document()->getEntry(getCurrentCategory(), curIndex, &d);
	PwM::copyToClipboard(d.url.c_str());
out:
	doc->timer()->putLock(DocTimer::id_autoLockTimer);
	doc->timer()->putLock(DocTimer::id_metaCheckTimer);
}

void PwMView::copyLauncherToClip()
{
	doc->timer()->getLock(DocTimer::id_metaCheckTimer);
	doc->timer()->getLock(DocTimer::id_autoLockTimer);
	unsigned int curIndex = 0;
	PwMDataItem d;
FIXME();
/*FIXME
	if (doc->isDeepLocked())
		goto out;
*/
	if (!getCurEntryIndex(&curIndex))
		goto out;
	document()->getEntry(getCurrentCategory(), curIndex, &d);
	PwM::copyToClipboard(d.launcher.c_str());
out:
	doc->timer()->putLock(DocTimer::id_autoLockTimer);
	doc->timer()->putLock(DocTimer::id_metaCheckTimer);
}

void PwMView::copyCommentToClip()
{
	doc->timer()->getLock(DocTimer::id_metaCheckTimer);
	doc->timer()->getLock(DocTimer::id_autoLockTimer);
	unsigned int curIndex = 0;
	PwMDataItem d;
FIXME();
/*FIXME
	if (doc->isDeepLocked())
		goto out;
*/
	if (!getCurEntryIndex(&curIndex))
		goto out;
	document()->getEntry(getCurrentCategory(), curIndex, &d);
	PwM::copyToClipboard(d.comment.c_str());
out:
	doc->timer()->putLock(DocTimer::id_autoLockTimer);
	doc->timer()->putLock(DocTimer::id_metaCheckTimer);
}

void PwMView::execLauncherAndCopy()
{
	copyPwToClip();
	mainClass->execLauncher_slot();
}
#endif

#include "pwmview.moc"
