/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004, 2005  by Michael Buesch                     *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef PWMVIEW_H_
#define PWMVIEW_H_

#include "pwmdoc.h"
#include "libpwmanager/libpwmanager.h"

#include <kconfig.h>
#include <klocale.h>
#include <klistview.h>

#include <qevent.h>
#include <qfont.h>
#include <qwidget.h>

#include <vector>
#include <string>

using std::string;
using std::vector;

class PwM;
class QScrollView;
class QSplitter;

/** Item of the entry tree. */
class TreeItem : public KListViewItem
{
public:
	TreeItem(QListView *parent,
		 DataItem *_data)
		 : KListViewItem (parent)
		 , data (_data)
		{ }
//TODO

protected:
	DataItem *data;
};

/** View class for PwManager Mainwindow */
class PwMView : public QWidget
{
	Q_OBJECT

public:
	/** construtor */
	PwMView(PwMDoc *_doc,
		QWidget *parent = 0,
		const char *name = 0);
	/** destructor */
	~PwMView();


	/** Returns a pointer to the document. */
	PwMDoc *getDoc()
			{ return doc; }

protected:
	/** Initialize the widget contents. */
	void initWidget();
	/** Widget resize event. */
	void resizeEvent(QResizeEvent *);

protected:
	/** document */
	PwMDoc *doc;
	/** Main splitter. */
	QSplitter *splitter;
	/** List view for the entry tree. */
	KListView *tree;
	/** Scroll view for the displayed items. */
	QScrollView *scroll;

#if 0
	/** returns pointer to the document */
	PwMDoc* document()
			{ return doc; }
	/** returns the index of the currently selected entry.
	  * (index as represented in PwMDoc !)
	  */
	bool getCurEntryIndex(unsigned int *index);
	/** returns the position of the given item in the document
	  * Note: This func only serches in the current category.
	  */
	bool getDocEntryIndex(unsigned int *index,
			      const QListViewItem *item);

//public slots:
	/** update the view (call if dirty) */
	void updateView();
	/** (re)sort all items and (re)shift them to listView. */
	void shiftToView();
	/** do the actual shifting. This is called from shiftToView() */
	void doShiftToView(QValueList< QPair<PwMDataItem, ShiftInfo> > &shiftList);
	/** handle clicking on an item */
	void handleToggle(QListViewItem *item);
	/** handle right-clicking on an item */
	void handleRightClick(QListViewItem *item, const QPoint &point, int);
	/** selects the item at "index" */
	void selAt(int index);
	/** rename category button pressed */
	void renCatButton_slot();
	/** delete category button pressed */
	void delCatButton_slot();

protected:
	/** right-click context-menu */
	QPopupMenu *ctxMenu;

protected:
	/** update the categories from document */
	void updateCategories();
	/** widget resize event */
	void resizeEvent(QResizeEvent *);
	/** initialize context-menu */
	void initCtxMenu();

//FIXME: remove
#if 0
	/** tempoarly disable auto-sorting and user-sorting */
	void tmpDisableSort()
			{ lv->setSorting(-1); }
	/** re-enable tempoarly disabled sorting */
	void tmpReEnableSort()
		{
			lv->setSorting(lv->columns() + 1,
				       true/*lv->sortOrder() == Qt::Ascending*/);
		}
#endif
	/** The user pressed and released a key. */
	void keyReleaseEvent(QKeyEvent *e);

//protected slots:
	/** changes the comment text-edit, because a new item has been selected */
	void refreshCommentTextEdit(QListViewItem *curItem);
	/** copy pw to clipboard */
	void copyPwToClip();
	/** copy name to clipboard */
	void copyNameToClip();
	/** copy desc to clipboard */
	void copyDescToClip();
	/** copy url to clipboard */
	void copyUrlToClip();
	/** copy launcher to clipboard */
	void copyLauncherToClip();
	/** copy comment to clipboard */
	void copyCommentToClip();
	/** execute the launcher and copy the pw to clipboard. */
	void execLauncherAndCopy();
	/** reorganize the "listViewPos" positions in the document
	  * (for the current category only!)
	  */
	void reorgLp();

private:
	/** document */
	PwMDoc *doc;
	/** pointer to the main class "PwM" */
	PwM *mainClass;
	/** Main splitter. */
	QSplitter *splitter;
	/** List view for the entry tree. */
	QListView *tree;
	/** Scroll view for the displayed items. */
	QScrollView *scroll;
#endif
};

#endif // PWMVIEW_H_
