/***************************************************************************
 *                                                                         *
 *   copyright (C) 2004 by Michael Buesch                                  *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This file contains code derived from                                  *
 *   pwsafe - commandline tool compatible with Counterpane's Passwordsafe  *
 *   Copyright (C) 2004 Nicolas S. Dade                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef PWSAFEFILE_H_
#define PWSAFEFILE_H_

#include "libpwmanager/sha1.h"
#include "libpwmanager/blowfish.h"
#include "pwmexception.h"

#include <qstring.h>

class PwMDoc;
class QFile;

/** class for importing/exporting data from/to
  * "pwsafe", "Counterpane's Passwordsafe" and "MyPasswordSafe".
  */
class PwSafeFile
{
protected:
	struct Header
	{
		unsigned char random[8];
		unsigned char hash[SHA1_HASH_LEN_BYTE];
		unsigned char salt[SHA1_HASH_LEN_BYTE];
		unsigned char iv[8];
	};

public:
	PwSafeFile();
	~PwSafeFile();

	/** import data from "filepath" into "doc" */
	bool importData(const QString &filepath,
			const QString &pw,
			PwMDoc *doc);
	/** export data from "doc" to "filepath" */
	bool exportData(const QString &filepath,
			const QString &pw,
			PwMDoc *doc);

protected:
	/** read the file header */
	bool readHeader(Header *hdr);
	/** write the file header */
	bool writeHeader(const Header &hdr);
	/** generate a new file header */
	void genNewHeader(Header *hdr,
			  const QString &pw);
	/** test the passphrase for correctness */
	bool testPw(const Header &hdr,
		    const QString &pw);
	/** hash/encrypt the password to the "hash"
	  * format used in the header.
	  */
	void hashPw(const QString &pw,
		    const unsigned char *headerRandom,
		    unsigned char *hash);
	/** import all entries in the file */
	bool importEntries(PwMDoc *doc,
			   const Header &hdr,
			   const QString &pw);
	/** do read an entry of the file */
	bool readEntry(Blowfish &bf,
		       unsigned char *ctx,
		       QString *ret);
	/** export all entries to the file */
	bool exportEntries(PwMDoc *doc,
			   const Header &hdr,
			   const QString &pw);
	/** do write an entry to the file */
	bool writeEntry(Blowfish &bf,
			unsigned char *ctx,
			const QString &e);

protected:
	/** file object for reading/writing */
	QFile *file;
#ifdef BIG_ENDIAN_HOST
	/** make a block of 8 bytes little endian */
	void makeBlockLE(unsigned char *block);
#else // BIG_ENDIAN_HOST
	void makeBlockLE(unsigned char *) { /* nothing */ }
#endif // BIG_ENDIAN_HOST
};

#endif // PWSAFEFILE_H_
